@@ -24,6 +24,7 @@ public abstract class Compilation extends CompilationInterface
   public String runtimeFile;
 
   public String sourcePath;
+  public String sourceEncoding;
   public String packagePath;
   public String destinationDir;
 
@@ -35,5 +36,8 @@ public abstract class Compilation extends CompilationInterface
 
   public java.util.Map packages;
   public java.util.Map javaTypeConstructors;
+
+  nice.tools.visibility.Scope javaScope;
+  public bossa.syntax.TypeScope globalTypeScope;
 }
 
