@@ -18,29 +18,18 @@ import bossa.util.*;
 /**
    A Scope level for variables.
 
-   @version $Date: 2005/01/14 18:53:54 $
+   @version $Date: 2005/03/12 02:41:47 $
    @author Daniel Bonniot (d.bonniot@mail.dotcom.fr)
 */
-public class VarScope
+public abstract class VarScope
 {
-  public VarScope(VarScope outer)
+  abstract void addSymbol(/*VarSymbol*/Symbol s);
+
+  void addSymbol(/*VarSymbol*/Symbol s, nice.tools.visibility.Visibility v)
   {
-    this.outer = outer;
-    this.defs = new HashMultiTable();
-  }
-  
-  public VarScope(VarScope outer, 
-		  Collection /* of VarSymbol */ defs)
-  {
-    this(outer);
-    addSymbols(defs);
+    this.addSymbol(s);
   }
 
-  void addSymbol(/*VarSymbol*/Symbol s)
-  {
-    this.defs.put(s.name,s);
-  }
-  
   /**
      Adds a collection of VarSymbols
    */
@@ -56,10 +45,46 @@ public class VarScope
 
       }
   }
-  
+
+  abstract void removeSymbol(/*VarSymbol*/Symbol sym);
+
+  public abstract List lookup(LocatedString i);
+
+  abstract List globalLookup(LocatedString i);
+
+  static VarScope create(VarScope outer)
+  {
+    return new LocalVarScope(outer);
+  }
+
+  static VarScope create(VarScope outer, Collection /* of VarSymbol */ defs)
+  {
+    return new LocalVarScope(outer, defs);
+  }
+}
+
+class LocalVarScope extends VarScope
+{
+  public LocalVarScope(VarScope outer)
+  {
+    this.outer = outer;
+    this.defs = new HashMap();
+  }
+
+  public LocalVarScope(VarScope outer, Collection /* of VarSymbol */ defs)
+  {
+    this(outer);
+    addSymbols(defs);
+  }
+
+  void addSymbol(/*VarSymbol*/Symbol s)
+  {
+    this.defs.put(s.name,s);
+  }
+
   void removeSymbol(/*VarSymbol*/Symbol sym)
   {
-    defs.remove(sym.name, sym);
+    defs.remove(sym.name);
   }
 
   /**
@@ -71,26 +96,38 @@ public class VarScope
    */
   public List lookup(LocatedString i)
   {
-    List res = defs.getAll(i);
-    
-    if(res!=null)
-      return res;
-    
+    Object res = defs.get(i);
+
+    if (res != null)
+      {
+        LinkedList l = new LinkedList();
+        l.add(res);
+        return l;
+      }
+
     if(outer!=null)
       return outer.lookup(i);
 
-    return new LinkedList();
+    return Collections.EMPTY_LIST;
+  }
+
+  List globalLookup(LocatedString i)
+  {
+    if (outer != null)
+      return outer.globalLookup(i);
+    else
+      return null;
   }
 
   /****************************************************************
    * Debugging
    ****************************************************************/
-  
+
   public String toString()
   {
-    return defs.elementCount()+";;\n"+outer;
+    return defs.size()+";;\n"+outer;
   }
-  
+
   private VarScope outer;
-  private HashMultiTable defs;
+  private HashMap defs;
 }
