@@ -17,7 +17,7 @@ import mlsub.typing.lowlevel.*;
 /**
    The kind of types qualified by their nullness.
 
-   @version $Date: 2004/06/16 10:00:36 $
+   @version $Date: 2005/06/18 09:58:39 $
    @author Daniel Bonniot (bonniot@users.sourceforge.net)
  */
 
@@ -37,25 +37,33 @@ public class NullnessKind implements AtomicKind
 
   public int arity() { return 1; }
 
-  private static TypeConstructor maybe, sure;
+  static TypeConstructor maybe, sure;
 
   public Monotype freshMonotype(boolean existential)
   {
     TypeConstructor tc = new TypeConstructor(instance);
     introduce(tc);
 
-    Monotype raw = new MonotypeVar(existential);
+    MonotypeVar raw = new MonotypeVar(existential);
     Typing.introduce(raw);
 
-    return new MonotypeConstructor(tc, new Monotype[]{ raw });
+    return new MonotypeConstructor(tc, new MonotypeVar[]{ raw });
   }
-  
-  public Monotype persistentFreshMonotype()
+
+  /**
+     @param base The type variable we create this fresh constructed
+                 monotype for.
+  */
+  public Monotype persistentFreshMonotype(MonotypeVar base)
   {
     TypeConstructor tc = new TypeConstructor(instance);
-    Monotype raw = new MonotypeVar();
 
-    return new MonotypeConstructor(tc, new Monotype[]{ raw });
+    /* It's important to give the raw variable the same name as the base one,
+       so that for the syntactic type <T> ... !T we don't end up printing !t9.
+    */
+    MonotypeVar raw = new MonotypeVar(base.getName());
+
+    return new MonotypeConstructor(tc, new MonotypeVar[]{ raw });
   }
   
   static void introduce(TypeConstructor tc)
@@ -84,23 +92,38 @@ public class NullnessKind implements AtomicKind
   public void leq(Element e1, Element e2)
     throws Unsatisfiable
   {
-    MonotypeConstructor m1 = mc(e1), m2 = mc(e2);
+    Monotype m1 = (Monotype) e1;
+    Monotype m2 = (Monotype) e2;
+
+    if (m1.isUnknown())
+      {
+        m2.setUnknown(false, true);
+        return;
+      }
+
+    if (m2.isUnknown())
+      {
+        m1.setUnknown(true, false);
+        return;
+      }
+
+    MonotypeConstructor mc1 = mc(m1), mc2 = mc(m2);
     
-    Engine.leq(m1.getTC(), m2.getTC());
-    Engine.leq(m1.getTP()[0], m2.getTP()[0]);
+    Engine.leq(mc1.getTC(), mc2.getTC());
+    Engine.leq(mc1.getTP()[0], mc2.getTP()[0]);
   }
   
-  private MonotypeConstructor mc(Element e)
+  private MonotypeConstructor mc(Monotype m)
   {
     try
       {
-	return (MonotypeConstructor) ((Monotype) e).equivalent();
+	return (MonotypeConstructor) m.equivalent();
       }
     catch(ClassCastException ex)
       {
 	throw new InternalError
-	  (e + " was expected to be a monotype constructor, " +
-	   " it's a " + e.getClass());
+	  (m + " was expected to be a monotype constructor, " +
+	   " it's a " + m.getClass());
       }
   }
   
