@@ -21,7 +21,7 @@ import mlsub.typing.lowlevel.Unsatisfiable;
 /**
    Static class for comparing types
 
-   @version $Date: 2004/03/24 23:44:24 $
+   @version $Date: 2004/06/29 18:38:18 $
    @author Daniel Bonniot
  */
 public final class Typing
@@ -664,5 +664,5 @@ public final class Typing
     return m.getKind() == TopMonotype.TopKind.instance;
   }
 
-  public static boolean dbg;
+  public static boolean dbg = bossa.util.Debug.typing;
 }
