@@ -6,7 +6,7 @@ package mlsub.typing.lowlevel;
    
    @see java.util.BitSet
    
-   @version $Revision: 1.10 $, $Date: 2004/04/17 17:12:11 $
+   @version $Revision: 1.13 $, $Date: 2005/03/30 23:08:11 $
    @author Alexandre Frey
    @author Daniel Bonniot 
    (replaced the underlying java.util.Vector by an array 
@@ -14,7 +14,7 @@ package mlsub.typing.lowlevel;
     most bitvectors are less than 64 elements, so using a long often avoids
     allocating the array)
  **/
-public class BitVector implements Cloneable, java.io.Serializable {
+public class BitVector implements java.io.Serializable {
   private final static int BITS_PER_UNIT = 6;
   private final static int MASK = (1<<BITS_PER_UNIT)-1;
 
@@ -49,7 +49,8 @@ public class BitVector implements Cloneable, java.io.Serializable {
    * Creates an empty set.
    */
   public BitVector() {
-    this(1 << BITS_PER_UNIT);
+    bits0 = 0L;
+    bits1 = null;
   }
 
   /**
@@ -80,6 +81,25 @@ public class BitVector implements Cloneable, java.io.Serializable {
       }
   }
 
+  /**
+   * Creates a copy of a BitVector.
+   */
+  public BitVector(BitVector old) {
+    if (old.bits1 == null)
+      {
+        this.bits0 = old.bits0;
+        this.bits1 = null;
+      }
+    else
+      {
+        this.bits0 = 0L;
+	// optim: shrink to nonZeroLength()?
+	int n = old.bits1.length;
+	this.bits1 = new long[n];
+	System.arraycopy(old.bits1, 0, this.bits1, 0, n);
+      }
+  }
+
   /**
    * Ensures that the BitVector can hold at least an nth bit.
    * This cannot leave the bits array at length 0.
@@ -520,31 +540,6 @@ public class BitVector implements Cloneable, java.io.Serializable {
     return true;
   }
 
-
-
-  /**
-   * Clones the BitVector.
-   */
-  final public Object clone() {
-    BitVector result = null;
-    try {
-      result = (BitVector) super.clone();
-    } catch (CloneNotSupportedException e) {
-      throw new InternalError("this shouldn't happen, since we are Cloneable");
-    }
-
-    if (bits1 == null)
-      result.bits0 = bits0;
-    else
-      {
-	// optim: shrink to nonZeroLength()?
-	int n = length();
-	result.bits1 = new long[n];
-	System.arraycopy(bits1, 0, result.bits1, 0, n);
-      }
-    return result;
-  }
-
   /**
    * Converts the BitVector to a String.
    */
@@ -794,9 +789,12 @@ public class BitVector implements Cloneable, java.io.Serializable {
    * Returns true if no bit is set in this BitVector
    **/
   public boolean isEmpty() {
-    int n = length();
+    if (bits1 == null)
+      return bits0 == 0L;
+
+    int n = bits1.length;
     for (int i = 0; i < n; i++) {
-      if (getW(i) != 0L) {
+      if (bits1[i] != 0L) {
         return false;
       }
     }
