@@ -14,7 +14,6 @@ package nice.tools.code;
 
 import gnu.bytecode.*;
 import gnu.expr.*;
-import bossa.syntax.MonoSymbol;
 
 /**
    Code generation functions.
@@ -28,9 +27,10 @@ public class Gen
 		new QuoteExp(ct));
   }
   
-  public static Expression isOfClass(Expression value, Type ct)
+  public static Expression isOfClass(Expression value, Type ct,
+                                     boolean possiblyNull)
   {
-    return Inline.inline(new IsOfClassProc(ct), value);
+    return Inline.inline(new IsOfClassProc(ct, possiblyNull), value);
   }
 
   public static Expression isNullExp(Expression value)
@@ -87,127 +87,6 @@ public class Gen
   private static final Expression equals = new QuoteExp
     (new PrimProcedure(Type.pointer_type.getDeclaredMethod("equals", 1)));
 
-  /**
-     Create a lambda expression to generate code for the method.
-
-     @param args can be null if there are no arguments
-  */
-  public static LambdaExp createMethod(String bytecodeName,
-				       Type[] argTypes,
-				       Type retType,
-				       MonoSymbol[] args)
-  {
-    return createMethod(bytecodeName, argTypes, retType, args, true, false);
-  }
-
-  /**
-     @param toplevel If the method can be called from foreign code.
-                     This forces its generation even if it is 
-		     apparently never called.
-  **/
-  public static LambdaExp createMethod(String bytecodeName,
-				       Type[] argTypes,
-				       Type retType,
-				       MonoSymbol[] args,
-				       boolean toplevel)
-  {
-    return createMethod(bytecodeName, argTypes, retType, args, toplevel, false);
-  }
-
-  /**
-     Create a lambda expression to generate code for the method.
-
-     @param args can be null if there are no arguments
-     @param member true iff this method is a non-static member of
-                   the class in argTypes[0]
-     @param toplevel If the method can be called from foreign code.
-                     This forces its generation even if it is 
-		     apparently never called.
-  **/
-  public static LambdaExp createMethod(String bytecodeName,
-				       Type[] argTypes,
-				       Type retType,
-				       MonoSymbol[] args,
-				       boolean toplevel,
-				       boolean member)
-  {
-    LambdaExp res = new LambdaExp();
-    createMethod(res, bytecodeName, argTypes, retType, args, toplevel, member, false);
-    return res;
-  }
-
-  public static ConstructorExp createConstructor
-    (Declaration thisDecl, Type[] argTypes, MonoSymbol[] args)
-  {
-    ConstructorExp res = new ConstructorExp(thisDecl);
-    createMethod(res, "<init>", argTypes, Type.void_type, args, 
-                 true, false, true);
-    return res;
-  }
-
-  public static ConstructorExp createCustomConstructor
-    (ClassType classType, Type[] argTypes, MonoSymbol[] args)
-  {
-    ConstructorExp res = new ConstructorExp(classType);
-    createMethod(res, "<init>", argTypes, Type.void_type, args, 
-                 true, false, true);
-    return res;
-  }
-
-  /**
-     Create a lambda expression to generate code for the method.
-
-     @param args can be null if there are no arguments
-     @param member true iff this method is a non-static member of
-                   the class in argTypes[0]
-     @param toplevel If the method can be called from foreign code.
-                     This forces its generation even if it is 
-		     apparently never called.
-  **/
-  private static void createMethod
-    (LambdaExp lexp, 
-     String bytecodeName,
-     Type[] argTypes,
-     Type retType,
-     MonoSymbol[] args,
-     boolean toplevel,
-     boolean member,
-     boolean constructor)
-  {
-    bytecodeName = nice.tools.code.Strings.escape(bytecodeName);
-    int arity = args == null ? 0 : args.length;
-
-    lexp.setReturnType(retType);
-    lexp.setName(bytecodeName);
-    lexp.min_args = lexp.max_args = member ? arity - 1 : arity;
-    lexp.forceGeneration();
-    if (toplevel)
-      lexp.setCanCall(true);
-    if (member)
-      lexp.setClassMethod(true);
-
-    // Parameters
-    for(int n = 0; n < arity; n++)
-      {
-	boolean isThis = member && n == 0;
-	String parameterName = args[n].getName() == null 
-	  ? "anonymous_" + n 
-	  : args[n].getName().toString();
-
-	gnu.expr.Declaration d;
-	if (isThis)
-	  {
-            d = new Declaration(parameterName);
-            d.context = lexp;
-          }
-	else
-	  d = lexp.addDeclaration(parameterName);
-	if (argTypes != null)
-	  d.setType(argTypes[n]);
-	d.noteValue(null);
-	args[n].setDeclaration(d, isThis);
-      }
-  }
 
   /**
      Create a lambda expression to generate code for the method.
@@ -226,7 +105,9 @@ public class Gen
      Type retType,
      Expression[] params)
   {
-    LambdaExp lexp = new LambdaExp();
+    Declaration thisDecl = new Declaration("this");
+    LambdaExp lexp = new MemberLambdaExp(thisDecl);
+
     bytecodeName = nice.tools.code.Strings.escape(bytecodeName);
     int arity = 1 + (argTypes == null ? 0 : argTypes.length);
 
@@ -246,8 +127,7 @@ public class Gen
 	gnu.expr.Declaration d;
 	if (isThis)
 	  {
-	    d = new Declaration(parameterName);
-	    d.context = lexp;
+	    d = thisDecl;
 	    d.setType(receiver);
 	    params[n] = new ThisExp(d);
 	  }
@@ -260,8 +140,8 @@ public class Gen
 	d.noteValue(null);
 	d.setCanRead(true);
 	d.setCanWrite(true);
-        
       }
+
     return lexp;
   }
 
@@ -270,18 +150,6 @@ public class Gen
     method.body = body;
   }
 
-  /**
-     @return an expression that references the lambda expression.
-  */
-  public static ReferenceExp referenceTo(LambdaExp lambda)
-  {
-    Declaration decl = new Declaration(lambda.getName());
-    decl.noteValue(lambda);
-    decl.setFlag(Declaration.IS_CONSTANT|Declaration.STATIC_SPECIFIED);
-    decl.setProcedureDecl(true);
-    return new ReferenceExp(decl);
-  }
-
   /**
      @return the lambda expression referenced.
   */
@@ -349,5 +217,16 @@ public class Gen
     
     lambda.body = new ApplyExp(proc, args);
     return lambda;
-  }  
+  }
+
+  public static Expression superCall(Expression method, Expression[] args)
+  {
+    return new ApplyExp(method, args, true);
+  }
+
+
+  public static Expression superCaller(Method method)
+  {
+    return new QuoteExp(PrimProcedure.specialCall(method));
+  }
 }
