@@ -15,7 +15,7 @@ package nice.tools.code;
 import mlsub.typing.*;
 import bossa.util.Debug;
 import bossa.util.Internal;
-import bossa.syntax.PrimitiveType;
+import nice.tools.typing.PrimitiveType;
 
 import gnu.bytecode.*;
 import gnu.expr.*;
@@ -25,7 +25,7 @@ import java.util.*;
 /**
    Conversion between Nice and bytecode types.
    
-   @version $Date: 2004/02/25 11:23:28 $
+   @version $Date: 2005/01/14 18:53:53 $
    @author Daniel Bonniot
  */
 
@@ -126,28 +126,6 @@ public final class Types
       setBytecodeType(ms[i]);
   }
 
-  public static void setBytecodeType(Polytype t)
-  {
-    Constraint cst = t.getConstraint();
-    if (Constraint.hasBinders(cst))
-      {
-	Typing.enter();
-	try{
-	  Constraint.enter(cst);
-	  setBytecodeType(t.getMonotype());
-	}
-	catch(TypingEx e) {}
-	finally{
-	  try{
-	    Typing.leave();
-	  }
-	  catch(TypingEx e) {}
-	}
-      }
-    else
-      setBytecodeType(t.getMonotype());
-  }
-
   /****************************************************************
    * Mapping monotypes to java types
    ****************************************************************/
@@ -295,8 +273,8 @@ public final class Types
   public static TypeConstructor typeConstructor (Type javaType)
     throws NotIntroducedClassException
   {
-    TypeConstructor tc = bossa.syntax.Node.getGlobalTypeScope().
-      globalLookup(javaType.getName(), null);
+    TypeConstructor tc = bossa.syntax.Node.globalTypeScopeLookup(
+	javaType.getName(), null);
 
     if (tc == null)
       {
@@ -318,7 +296,7 @@ public final class Types
     int len = javaTypes.length;
     Monotype[] res = new Monotype[len];
     for (int i = 0; i < len; i++)
-      res[i] = monotype(javaTypes[i], typeVariables, niceTypeVariables);
+      res[i] = monotype(javaTypes[i], true, typeVariables, niceTypeVariables);
     return res;
   }
 
@@ -328,70 +306,31 @@ public final class Types
     return monotype(javaType, sure, null, null);
   }
 
+  /**
+    Special version for use in Import.java,
+    it always checks whether nullness info may be added.
+  */
   public static Monotype monotype(Type javaType, boolean sure,
 				  TypeVariable[] typeVariables,
 				  TypeSymbol[] niceTypeVariables)
     throws ParametricClassException, NotIntroducedClassException
-  {
-    return monotype(javaType, sure, typeVariables, niceTypeVariables, false);
-  }
-
-  public static Monotype monotype(Type javaType)
-    throws ParametricClassException, NotIntroducedClassException
-  {
-    return monotype(javaType, null, null);
-  }
-
-  public static Monotype monotype(Type javaType,
-				  TypeVariable[] typeVariables,
-				  TypeSymbol[] niceTypeVariables)
-    throws ParametricClassException, NotIntroducedClassException
   {
     Monotype res = getMonotype(javaType, typeVariables, niceTypeVariables);
-    if (javaType instanceof ObjectType &&
-	! (javaType instanceof TypeVariable))
-      return bossa.syntax.Monotype.maybe(res);
-    else
-      // the sure is already there in getMonotype
-      return res;
-  }
-
-  /**
-    Special version for use in Import.java,
-    it always checks whether nullness info may be added.
-    And can handle arrays different(not yet implemented)
-  */
-  public static Monotype monotype(Type javaType, boolean sure,
-				  TypeVariable[] typeVariables,
-				  TypeSymbol[] niceTypeVariables,
-				  boolean arraySure)
-    throws ParametricClassException, NotIntroducedClassException
-  {
-    Monotype res = getMonotype(javaType, typeVariables, niceTypeVariables,
-			       arraySure);
     //primitivetypes and typevariables should not get nullness info
     if (javaType instanceof ObjectType &&
 	! (javaType instanceof TypeVariable))
       {
 	if (sure)
-	  return bossa.syntax.Monotype.sure(res);
+	  return nice.tools.typing.Types.sureMonotype(res);
     	else
-	  return bossa.syntax.Monotype.maybe(res);
+	  return nice.tools.typing.Types.maybeMonotype(res);
       }
     return res;
   }
-  private static Monotype getMonotype(Type javaType,
-				      TypeVariable[] typeVariables,
-				      TypeSymbol[] niceTypeVariables)
-    throws ParametricClassException, NotIntroducedClassException
-  {
-    return getMonotype(javaType, typeVariables, niceTypeVariables, false);
-  }
 
   private static Monotype getMonotype(Type javaType,
 				      TypeVariable[] typeVariables,
-				      TypeSymbol[] niceTypeVariables,
-				      boolean arraySure)
+				      TypeSymbol[] niceTypeVariables)
     throws ParametricClassException, NotIntroducedClassException
   {
     if(javaType.isVoid())
@@ -414,22 +353,12 @@ public final class Types
       return PrimitiveType.doubleType;
 
     if (javaType instanceof ArrayType)
-      if (arraySure)
-	//making the component of an array sure
-	return new MonotypeConstructor
-	  (PrimitiveType.arrayTC, 
+      return new MonotypeConstructor(PrimitiveType.arrayTC, 
 	   new Monotype[]{
 	     monotype(((ArrayType) javaType).getComponentType(), true, 
-		      typeVariables, niceTypeVariables, true)
+		      typeVariables, niceTypeVariables)
 	   });
-      else
-	return new MonotypeConstructor
-	(PrimitiveType.arrayTC, 
-	 new Monotype[]{
-	   monotype(((ArrayType) javaType).getComponentType(), 
-		    typeVariables, niceTypeVariables)
-	 });
-    
+
     if (javaType instanceof ParameterizedType)
       {
 	ParameterizedType p = (ParameterizedType) javaType;
@@ -460,7 +389,23 @@ public final class Types
     if (javaType == Type.pointer_type)
       return TopMonotype.instance;
 
-    return getMonotype(javaType.getName());
+    TypeConstructor tc = bossa.syntax.Node.globalTypeScopeLookup(javaType.getName(), null);
+
+    if (tc == null)
+      {
+	Internal.warning(javaType.getName() + " is not known");
+	throw new NotIntroducedClassException(tc);
+      }
+
+    if (tc.getId() == -1)
+      throw new NotIntroducedClassException(tc);
+
+    try {
+      return nice.tools.typing.Types.unknownArgsMonotype(tc);
+    }
+    catch (BadSizeEx ex) {
+      throw new ParametricClassException(tc.toString());
+    }
   }
   
   /** 
@@ -492,35 +437,6 @@ public final class Types
     }
   }
 
-  private static Monotype getMonotype(String name)
-  throws ParametricClassException, NotIntroducedClassException
-  {
-    if(name.endsWith("[]"))
-      {
-	name=name.substring(0,name.length()-2);
-	return new MonotypeConstructor
-	  (PrimitiveType.arrayTC, 
-	   new Monotype[]{ getMonotype(name) });
-      }
-    
-    TypeConstructor tc = bossa.syntax.Node.getGlobalTypeScope().
-      globalLookup(name, null);
-
-    if (tc == null)
-      {
-	Internal.warning(name + " is not known");
-	throw new NotIntroducedClassException(tc);
-      }
-
-    if (tc.variance != null && tc.arity() != 0)
-      throw new ParametricClassException(tc.toString());
-
-    if (tc.getId() == -1)
-      throw new NotIntroducedClassException(tc);
-	
-    return new MonotypeConstructor(tc, null);
-  }
-  
   /****************************************************************
    * Converting string to gnu.bytecode.Type
    ****************************************************************/
@@ -556,12 +472,12 @@ public final class Types
     
     return TypeImport.lookup(s, loc);
   }
-  
+
   /**
-     The type is not necessarily fully qulified. 
+     The type is not necessarily fully qualified.
   */
-  public static final 
-  gnu.bytecode.Type typeRepresentationToBytecode(String type, 
+  public static final
+  gnu.bytecode.Type typeRepresentationToBytecode(String type,
 						 bossa.util.Location loc)
   {
     if(type.charAt(0)=='[')
@@ -572,9 +488,11 @@ public final class Types
 	else
           return SpecialArray.create(res);
       }
-    
-    TypeConstructor sym = bossa.syntax.Node.getGlobalTypeScope().
-      globalLookup(type, loc);
+
+    if (type.equals("Object") || type.equals("java.lang.Object"))
+      return Type.pointer_type;
+
+    TypeConstructor sym = bossa.syntax.Node.globalTypeScopeLookup(type, loc);
 
     return get(sym);
   }
@@ -601,40 +519,6 @@ public final class Types
       ("Equivalent type for " + t + " is not defined yet");
     return null;
   }
-  
-  /****************************************************************
-   * Default values
-   ****************************************************************/
-
-  public static Expression defaultValue(Monotype m)
-  {
-    if (!(m instanceof MonotypeConstructor))
-      return QuoteExp.nullExp;
-    
-    TypeConstructor tc = nice.tools.typing.Types.rawType(m).head();
-
-    if (tc == null)
-      return QuoteExp.nullExp;
-
-    if(tc == PrimitiveType.intTC ||
-       tc == PrimitiveType.byteTC ||
-       tc == PrimitiveType.shortTC ||
-       tc == PrimitiveType.longTC)
-      return zeroInt;
-    if(tc == PrimitiveType.floatTC ||
-       tc == PrimitiveType.doubleTC)
-      return zeroFloat;
-    if(tc == PrimitiveType.boolTC)
-      return QuoteExp.falseExp;
-    if(tc == PrimitiveType.charTC)
-      return zeroChar;
-    
-    return QuoteExp.nullExp;
-  }
-
-  private static Expression zeroInt = new QuoteExp(new Integer(0));
-  private static Expression zeroFloat = new QuoteExp(new Float(0.0));
-  private static Expression zeroChar = new QuoteExp(new Character((char) 0));
 
   /****************************************************************
    * Reset the state for a new compilation.
@@ -643,7 +527,18 @@ public final class Types
   public static void reset()
   {
     tcToGBType = new HashMap();
-    TypeImport.stringToReflectClass = new HashMap();
     Import.reset();
   }
+
+  /**
+   * Test if use of a TypeConstructor is legal within a given package
+   */
+  public static boolean legalAccess(TypeConstructor tc, String packageName )
+  {
+    Type type = get(tc);
+    if (! (type instanceof ClassType))
+      return true;
+
+    return Access.legal((ClassType)type, packageName);
+  }
 }
