@@ -98,7 +98,7 @@ public final class Types
   public static Monotype rawType(Monotype m)
   {
     m = m.equivalent();
-    if (! (m.getKind() == NullnessKind.instance))
+    if (m.getKind() != NullnessKind.instance || m == UnknownMonotype.instance)
       return m;
 
     return ((MonotypeConstructor) m).getTP()[0];
@@ -228,7 +228,7 @@ public final class Types
 
     Monotype[] parameters = ((MonotypeConstructor) type).getTP();
 
-    if (parameters.length <= rank)
+    if (parameters == null || parameters.length <= rank)
       return null;
     else
       return parameters[rank];
@@ -305,6 +305,41 @@ public final class Types
     return true;
   }
 
+  /**
+     @returns true if functional types t1 and t2 have a partly common domain.
+              that is, there exists some types that belong to both domains.
+  */
+  public static boolean domainsIntersect(Polytype t1, Polytype t2)
+  {
+    Typing.enter();
+
+    try {
+
+      try {
+
+        Constraint.enter(t1.getConstraint());
+        Constraint.enter(t2.getConstraint());
+
+        // There exists argument types ...
+        Monotype[] args = MonotypeVar.news(parameters(t2).length);
+        Typing.introduce(args);
+
+        // ... that can be used for both methods ...
+        Typing.leq(args, parameters(t1));
+        Typing.leq(args, parameters(t2));
+      }
+      finally {
+        Typing.leave();
+      }
+    }
+    catch (TypingEx ex) {
+      return false;
+    }
+
+    // OK, there is an intersection
+    return true;
+  }
+
   /**
      @returns true if the spec type specializes type parameters of the original
        type (which can not be checked at runtime during dispatch, and therefore
@@ -353,4 +388,71 @@ public final class Types
     // OK, no covariant dispatch
     return false;
   }
+
+  /****************************************************************
+   * Merging
+   ****************************************************************/
+
+  // <bonniot> this merge is not optimal since it does not search of a common supertype,
+  // <bonniot> only if one is smaller than the other
+  // <bonniot> but it should be useful already, and enough to test an algo that needs it
+  // <arjanb> yeah it would handle 95% of the cases
+  public static Monotype merge(Monotype m1, Monotype m2)
+  {
+    if (m1 == m2)
+      return m1;
+
+    Monotype raw = rawMerge(equivalent(m1), equivalent(m2));
+    if (raw == null)
+      return null;
+
+    TypeConstructor marker;
+    if (isSure(m1) && isSure(m2))
+      marker = PrimitiveType.sureTC;
+    else
+      marker = PrimitiveType.maybeTC;
+
+    Monotype res = new MonotypeConstructor(marker, new Monotype[]{raw});
+    return res;
+  }
+
+  private static Monotype rawMerge(Monotype raw1, Monotype raw2)
+  {
+    if (raw1 == raw2)
+      return raw1;
+
+    if (raw1 == TopMonotype.instance || raw2 == TopMonotype.instance)
+      return TopMonotype.instance;
+
+    TypeConstructor head1 = raw1.head();
+    TypeConstructor head2 = raw2.head();
+
+    TypeConstructor head;
+    if (Typing.testRigidLeq(raw1.head(), raw2.head()))
+      head = raw2.head();
+    else if (Typing.testRigidLeq(raw2.head(), raw1.head()))
+      head = raw1.head();
+    else
+      return null;
+
+    Monotype[] args1 = ((MonotypeConstructor) raw1).getTP();
+    Monotype[] args2 = ((MonotypeConstructor) raw2).getTP();
+    Monotype[] args;
+    if (args1 == null && args2 == null)
+      // no-arg type constructors
+      args = null;
+    else
+      {
+        // Resursively merge the type parameters.
+        args = new Monotype[args1.length];
+        for (int i = 0; i < args.length; i++)
+          {
+            args[i] = merge(args1[i], args2[i]);
+            if (args[i] == null)
+              return null;
+          }
+      }
+
+    return new MonotypeConstructor(head, args);
+  }
 }
